@@ -6,6 +6,7 @@ import ipaddress
 import os
 import pathlib
 import re
+import sys
 import types
 import typing
 import uuid
@@ -27,6 +28,7 @@ from fractions import Fraction
 from typing import Any, ForwardRef, Optional, Tuple, Union
 
 import typing_extensions
+from typing_extensions import NotRequired
 
 from mashumaro.core.const import PY_311_MIN
 from mashumaro.core.helpers import parse_timezone
@@ -36,6 +38,7 @@ from mashumaro.core.meta.helpers import (
     get_class_that_defines_method,
     get_function_arg_annotation,
     get_literal_values,
+    get_type_origin,
     get_type_var_default,
     is_final,
     is_generic,
@@ -44,6 +47,7 @@ from mashumaro.core.meta.helpers import (
     is_new_type,
     is_not_required,
     is_optional,
+    is_readonly,
     is_required,
     is_self,
     is_special_typing_primitive,
@@ -91,6 +95,13 @@ from mashumaro.types import (
     SerializationStrategy,
 )
 
+if sys.version_info >= (3, 14):
+    from typing import evaluate_forward_ref
+
+    from annotationlib import get_annotations
+else:
+    from typing_extensions import evaluate_forward_ref, get_annotations
+
 try:
     import ciso8601
 except ImportError:  # pragma: no cover
@@ -560,9 +571,7 @@ def _unpack_with_annotated_serialization_strategy(
     except (KeyError, ValueError):
         value_type = Any
     if isinstance(value_type, ForwardRef):
-        value_type = spec.builder.evaluate_forward_ref(
-            value_type, spec.origin_type
-        )
+        value_type = evaluate_forward_ref(value_type)
     value_type = substitute_type_params(
         value_type,  # type: ignore
         resolve_type_params(strategy_type, get_args(spec.type))[strategy_type],
@@ -648,9 +657,7 @@ def _unpack_annotated_serializable_type(
             f"._deserialize({spec.expression})"
         )
     if isinstance(value_type, ForwardRef):
-        value_type = spec.builder.evaluate_forward_ref(
-            value_type, spec.origin_type
-        )
+        value_type = evaluate_forward_ref(value_type)
     value_type = substitute_type_params(
         value_type,
         resolve_type_params(spec.origin_type, get_args(spec.type))[
@@ -866,13 +873,13 @@ def unpack_special_typing_primitive(spec: ValueSpec) -> Optional[Expression]:
         elif is_type_var_tuple(spec.type):
             return UnpackerRegistry.get(spec.copy(type=tuple[Any, ...]))
         elif isinstance(spec.type, ForwardRef):
-            evaluated = spec.builder.evaluate_forward_ref(
-                spec.type, spec.owner
-            )
+            evaluated = evaluate_forward_ref(spec.type)
             if evaluated is not None:
                 return UnpackerRegistry.get(spec.copy(type=evaluated))
         elif is_type_alias_type(spec.type):
             return UnpackerRegistry.get(spec.copy(type=spec.type.__value__))
+        elif is_readonly(spec.type):
+            return UnpackerRegistry.get(spec.copy(type=get_args(spec.type)[0]))
         raise UnserializableDataError(
             f"{spec.type} as a field type is not supported by mashumaro"
         )
@@ -1060,7 +1067,7 @@ def unpack_named_tuple(spec: ValueSpec) -> Expression:
     ]
     annotations = {
         k: resolved.get(v, v)
-        for k, v in getattr(spec.origin_type, "__annotations__", {}).items()
+        for k, v in get_annotations(spec.origin_type, eval_str=True).items()
     }
     fields = getattr(spec.type, "_fields", ())
     defaults = getattr(spec.type, "_field_defaults", {})
@@ -1148,11 +1155,20 @@ def unpack_typed_dict(spec: ValueSpec) -> Expression:
     ]
     annotations = {
         k: resolved.get(v, v)
-        for k, v in spec.origin_type.__annotations__.items()
+        for k, v in get_annotations(spec.origin_type, eval_str=True).items()
     }
     all_keys = list(annotations.keys())
-    required_keys = getattr(spec.type, "__required_keys__", all_keys)
-    optional_keys = getattr(spec.type, "__optional_keys__", [])
+    required_keys = set(getattr(spec.type, "__required_keys__", all_keys))
+    optional_keys = set(getattr(spec.type, "__optional_keys__", []))
+
+    # workaround for https://github.com/python/cpython/issues/97727
+    for key, annotation in annotations.items():
+        if isinstance(annotation, ForwardRef):
+            annotation = evaluate_forward_ref(annotation)
+            if get_type_origin(annotation) is NotRequired:
+                required_keys.discard(key)
+                optional_keys.add(key)
+
     lines = CodeLines()
     method_name = (
         f"__unpack_typed_dict_{spec.builder.cls.__name__}_"
