@@ -1,10 +1,11 @@
 import datetime
 import ipaddress
 import os
+import sys
 import warnings
 from base64 import encodebytes
 from collections import ChainMap, Counter, deque
-from collections.abc import (
+from collections.abc import (  # type: ignore[attr-defined]
     ByteString,
     Callable,
     Collection,
@@ -22,15 +23,13 @@ from typing import Any, ForwardRef, Optional, Tuple, Type, Union
 from uuid import UUID
 from zoneinfo import ZoneInfo
 
-from typing_extensions import TypeAlias
+from typing_extensions import NotRequired, TypeAlias
 
 from mashumaro.config import BaseConfig
 from mashumaro.core.const import PY_311_MIN
 from mashumaro.core.meta.code.builder import CodeBuilder
 from mashumaro.core.meta.helpers import (
-    evaluate_forward_ref,
     get_args,
-    get_forward_ref_referencing_globals,
     get_function_return_annotation,
     get_literal_values,
     get_type_origin,
@@ -52,7 +51,7 @@ from mashumaro.core.meta.helpers import (
     resolve_type_params,
     type_name,
 )
-from mashumaro.core.meta.types.common import NoneType
+from mashumaro.core.meta.types.common import NoneType, clean_id
 from mashumaro.helper import pass_through
 from mashumaro.jsonschema.annotations import (
     Annotation,
@@ -94,6 +93,13 @@ try:
 except ImportError:  # pragma: no cover
     from mashumaro.mixins.json import DataClassJSONMixin  # type: ignore
 
+if sys.version_info >= (3, 14):
+    from typing import evaluate_forward_ref
+
+    from annotationlib import get_annotations
+else:
+    from typing_extensions import evaluate_forward_ref, get_annotations
+
 
 UTC_OFFSET_PATTERN = r"^UTC([+-][0-2][0-9]:[0-5][0-9])?$"
 
@@ -143,14 +149,11 @@ class Instance:
     def derive(self, **changes: Any) -> "Instance":
         new_type = changes.get("type")
         if isinstance(new_type, ForwardRef):
-            changes["type"] = evaluate_forward_ref(
-                new_type,
-                get_forward_ref_referencing_globals(new_type, self.type),
-                self.__dict__,
-            )
+            changes["type"] = evaluate_forward_ref(new_type)
         new_instance = replace(self, **changes)
         if is_dataclass(self.origin_type):
             new_instance.__owner_builder = self.__self_builder
+            new_instance.update_type(new_instance.type)
         return new_instance
 
     def __post_init__(self) -> None:
@@ -296,7 +299,9 @@ def _get_schema_or_none(
     return schema
 
 
-def _default(f_type: Type, f_value: Any, config_cls: Type[BaseConfig]) -> Any:
+def _default(
+    f_type: Optional[Type], f_value: Any, config_cls: Type[BaseConfig]
+) -> Any:
     @dataclass
     class CC(DataClassJSONMixin):
         x: f_type = f_value  # type: ignore
@@ -351,9 +356,14 @@ def on_type_with_overridden_serialization(
 def on_dataclass(instance: Instance, ctx: Context) -> Optional[JSONSchema]:
     # TODO: Self references might not work
     if is_dataclass(instance.origin_type):
+        if ctx.all_refs:
+            title = clean_id(type_name(instance.type, short=True))
+            title = title.strip("_")
+        else:
+            title = instance.origin_type.__name__
         jsonschema_config = instance.get_self_config().json_schema
         schema = JSONObjectSchema(
-            title=instance.origin_type.__name__,
+            title=title,
             additionalProperties=jsonschema_config.get(
                 "additionalProperties", False
             ),
@@ -385,11 +395,9 @@ def on_dataclass(instance: Instance, ctx: Context) -> Optional[JSONSchema]:
         if required:
             schema.required = required
         if ctx.all_refs:
-            ctx.definitions[instance.origin_type.__name__] = schema
+            ctx.definitions[title] = schema
             ref_prefix = ctx.ref_prefix or ctx.dialect.definitions_root_pointer
-            return JSONSchema(
-                reference=f"{ref_prefix}/{instance.origin_type.__name__}"
-            )
+            return JSONSchema(reference=f"{ref_prefix}/{title}")
         else:
             return schema
 
@@ -461,11 +469,7 @@ def on_special_typing_primitive(
     elif is_readonly(instance.type):
         return get_schema(instance.derive(type=args[0]), ctx)
     elif isinstance(instance.type, ForwardRef):
-        evaluated = evaluate_forward_ref(
-            instance.type,
-            get_forward_ref_referencing_globals(instance.type),
-            None,
-        )
+        evaluated = evaluate_forward_ref(instance.type)
         if evaluated is not None:
             return get_schema(instance.derive(type=evaluated), ctx)
 
@@ -639,8 +643,8 @@ def on_named_tuple(instance: Instance, ctx: Context) -> JSONSchema:
     )[instance.origin_type]
     annotations = {
         k: resolved.get(v, v)
-        for k, v in getattr(
-            instance.origin_type, "__annotations__", {}
+        for k, v in get_annotations(
+            instance.origin_type, eval_str=True
         ).items()
     }
     fields = getattr(instance.type, "_fields", ())
@@ -685,10 +689,20 @@ def on_typed_dict(instance: Instance, ctx: Context) -> JSONObjectSchema:
     )[instance.origin_type]
     annotations = {
         k: resolved.get(v, v)
-        for k, v in instance.origin_type.__annotations__.items()
+        for k, v in get_annotations(
+            instance.origin_type, eval_str=True
+        ).items()
     }
     all_keys = list(annotations.keys())
-    required_keys = getattr(instance.type, "__required_keys__", all_keys)
+    required_keys = set(getattr(instance.type, "__required_keys__", all_keys))
+
+    # workaround for https://github.com/python/cpython/issues/97727
+    for key, annotation in annotations.items():
+        if isinstance(annotation, ForwardRef):
+            annotation = evaluate_forward_ref(annotation)
+            if get_type_origin(annotation) is NotRequired:
+                required_keys.discard(key)
+
     return JSONObjectSchema(
         properties={
             key: get_schema(instance.derive(type=annotations[key]), ctx)
