@@ -32,6 +32,7 @@ from typing import (
     OrderedDict,
     Sequence,
     Tuple,
+    TypeVar,
     Union,
 )
 from uuid import UUID
@@ -143,7 +144,7 @@ class DataClassWithThirdPartyType:
 
 def test_jsonschema_for_dataclass():
     @dataclass
-    class DataClass:
+    class MyClass:
         a: int
         b: float = 3.14
         c: Optional[int] = field(default=None, metadata={"alias": "cc"})
@@ -156,8 +157,8 @@ def test_jsonschema_for_dataclass():
         class Config:
             aliases = {"a": "aa", "d": "dd"}
 
-    schema = JSONObjectSchema(
-        title="DataClass",
+    assert build_json_schema(MyClass) == JSONObjectSchema(
+        title="MyClass",
         properties={
             "aa": JSONSchema(type=JSONSchemaInstanceType.INTEGER),
             "b": JSONSchema(type=JSONSchemaInstanceType.NUMBER, default=3.14),
@@ -177,9 +178,35 @@ def test_jsonschema_for_dataclass():
         additionalProperties=False,
         required=["aa"],
     )
-    assert build_json_schema(DataClass) == schema
-    assert build_json_schema(DataClass, all_refs=True) == JSONSchema(
-        reference="#/$defs/DataClass", definitions={"DataClass": schema}
+    assert build_json_schema(MyClass, all_refs=True) == JSONSchema(
+        reference="#/$defs/test_jsonschema_for_dataclass__locals__MyClass",
+        definitions={
+            "test_jsonschema_for_dataclass__locals__MyClass": JSONObjectSchema(
+                title="test_jsonschema_for_dataclass__locals__MyClass",
+                properties={
+                    "aa": JSONSchema(type=JSONSchemaInstanceType.INTEGER),
+                    "b": JSONSchema(
+                        type=JSONSchemaInstanceType.NUMBER, default=3.14
+                    ),
+                    "cc": JSONSchema(
+                        anyOf=[
+                            JSONSchema(type=JSONSchemaInstanceType.INTEGER),
+                            JSONSchema(type=JSONSchemaInstanceType.NULL),
+                        ],
+                        default=None,
+                    ),
+                    "dd": JSONSchema(
+                        type=JSONSchemaInstanceType.STRING, default=""
+                    ),
+                    "f": JSONArraySchema(
+                        items=JSONSchema(type=JSONSchemaInstanceType.INTEGER),
+                        description="description for f",
+                    ),
+                },
+                additionalProperties=False,
+                required=["aa"],
+            )
+        },
     )
 
 
@@ -1156,7 +1183,12 @@ def test_jsonschema_with_ref_prefix():
     class DataClass:
         pass
 
-    schema = {"$ref": "#/components/responses/DataClass"}
+    schema = {
+        "$ref": (
+            "#/components/responses/"
+            "test_jsonschema_with_ref_prefix__locals__DataClass"
+        )
+    }
     assert (
         build_json_schema(
             List[DataClass], all_refs=True, ref_prefix="#/components/responses"
@@ -1443,3 +1475,72 @@ def test_jsonschema_with_custom_instance_format():
         required=["x", "y"],
         additionalProperties=False,
     )
+
+
+def test_jsonschema_for_generic_dataclass():
+    T = TypeVar("T")
+
+    @dataclass
+    class MyClass(Generic[T]):
+        x: T
+        y: list[T]
+
+    assert build_json_schema(MyClass) == JSONObjectSchema(
+        title="MyClass",
+        properties={
+            "x": EmptyJSONSchema(),
+            "y": JSONArraySchema(),
+        },
+        additionalProperties=False,
+        required=["x", "y"],
+    )
+    assert build_json_schema(MyClass[int]) == JSONObjectSchema(
+        title="MyClass",
+        properties={
+            "x": JSONSchema(type=JSONSchemaInstanceType.INTEGER),
+            "y": JSONArraySchema(
+                items=JSONSchema(type=JSONSchemaInstanceType.INTEGER)
+            ),
+        },
+        additionalProperties=False,
+        required=["x", "y"],
+    )
+
+    @dataclass
+    class MyClass2(Generic[T]):
+        z: MyClass[T]
+
+    assert build_json_schema(MyClass2) == JSONObjectSchema(
+        title="MyClass2",
+        properties={
+            "z": JSONObjectSchema(
+                title="MyClass",
+                properties={
+                    "x": EmptyJSONSchema(),
+                    "y": JSONArraySchema(),
+                },
+                additionalProperties=False,
+                required=["x", "y"],
+            )
+        },
+        additionalProperties=False,
+        required=["z"],
+    )
+    assert build_json_schema(MyClass2[str]) == JSONObjectSchema(
+        title="MyClass2",
+        properties={
+            "z": JSONObjectSchema(
+                title="MyClass",
+                properties={
+                    "x": JSONSchema(type=JSONSchemaInstanceType.STRING),
+                    "y": JSONArraySchema(
+                        items=JSONSchema(type=JSONSchemaInstanceType.STRING)
+                    ),
+                },
+                additionalProperties=False,
+                required=["x", "y"],
+            )
+        },
+        additionalProperties=False,
+        required=["z"],
+    )
